"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fs = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const util_1 = tslib_1.__importDefault(require("util"));
const path_1 = tslib_1.__importDefault(require("path"));
const utils_1 = require("./utils");
const ENOENT = 'ENOENT'; // not found
const EEXIST = 'EEXIST';
function copy(src, dist, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        opts = Object.assign({ overwrite: true }, opts);
        const srcStat = yield fs_1.default.promises.stat(src);
        const isSkiped = opts.skip ? yield opts.skip(src, srcStat) : false;
        if (isSkiped)
            return;
        // auto merge for directory
        if (srcStat.isDirectory()) {
            yield fsExtra.mkdirp(dist);
            let childs = yield fs_1.default.promises.readdir(src);
            yield Promise.all(childs.map((child) => copy(path_1.default.join(src, child), path_1.default.join(dist, child), opts)));
        }
        else if (srcStat.isFile() || srcStat.isSymbolicLink()) {
            if (yield fsExtra.lexists(dist)) {
                let isDir = yield fsExtra.isDirectory(dist);
                if (isDir) {
                    dist = path_1.default.join(dist, path_1.default.basename(src));
                }
                else if (opts.overwrite) {
                    yield fsExtra.rmrf(dist);
                }
                else {
                    return;
                }
            }
            else {
                let lastChar = dist[dist.length - 1];
                if (lastChar === '/' || lastChar === '\\') {
                    yield fsExtra.mkdirp(dist);
                    dist = path_1.default.join(dist, path_1.default.basename(src));
                }
                else {
                    let dir = path_1.default.dirname(dist);
                    yield fsExtra.mkdirp(dir);
                }
            }
            yield fsExtra.copyFile(src, dist);
        }
    });
}
function watchDir(dir, options, cb) {
    if (utils_1.Is.fn(options)) {
        cb = options;
        options = void 0;
    }
    options = Object.assign({ persistent: true, threshold: 300 }, options);
    cb = cb && (0, utils_1.promiseQueue)(cb);
    if (options.threshold) {
        cb = cb && (0, utils_1.debounce)(cb, options.threshold);
    }
    if (process.platform === 'linux' && options.iterOnLinux) {
        // tslint:disable-next-line:no-floating-promises
        fsExtra.iter(dir, (file) => {
            fs_1.default.watch(file, {
                recursive: false,
                persistent: options.persistent,
            }, cb);
        });
    }
    else {
        fs_1.default.watch(dir, { recursive: true, persistent: options.persistent }, cb);
    }
}
const fsExtra = {
    watchDir,
    copy,
    exists(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield fs_1.default.promises.stat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
            return true;
        });
    },
    /** exists via lstat, if a symbolic link's target file doesn't exists, `fs.exists` will return false, but `fs.lexists` will return true. */
    lexists(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield fs_1.default.promises.lstat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
            return true;
        });
    },
    isFile(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield fs_1.default.promises.lstat(path)).isFile();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    },
    isDirectory(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield fs_1.default.promises.lstat(path)).isDirectory();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    },
    isSymbolicLink(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield fs_1.default.promises.lstat(path)).isSymbolicLink();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    },
    copyFile: fs_1.default.copyFile
        ? util_1.default.promisify(fs_1.default.copyFile)
        : (src, dist) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            yield fs_1.default.promises.stat(src);
            return new Promise((res, rej) => {
                fs_1.default.createReadStream(src, { highWaterMark: 2 * 1024 * 1024 })
                    .pipe(fs_1.default.createWriteStream(dist))
                    .on('error', rej)
                    .on('close', res);
            });
        }),
    /**
     * Make directory with parents, like `mkdir -p`
     * @param dir
     */
    mkdirp(dir) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parent = path_1.default.dirname(dir);
            if (!(yield fsExtra.exists(parent))) {
                yield fsExtra.mkdirp(parent);
            }
            return fs_1.default.promises.mkdir(dir).catch((error) => {
                if (error.code !== EEXIST) {
                    throw error;
                }
            });
        });
    },
    /**
     * Make directory with parents, like `mkdir -p`
     * @param dir
     */
    mkdirpSync(dir) {
        if (dir === '/')
            return;
        let parent = path_1.default.dirname(dir);
        if (!fs_1.default.existsSync(parent)) {
            fsExtra.mkdirpSync(parent);
        }
        if (!fs_1.default.existsSync(dir)) {
            try {
                fs_1.default.mkdirSync(dir);
            }
            catch (error) {
                if (error.code !== EEXIST) {
                    throw error;
                }
            }
        }
    },
    /**
     * Remove file or directory recursively, like `rm -rf`
     * @param path The path to remove
     * @param opts Options
     */
    rmrf(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let stat;
            try {
                stat = yield fs_1.default.promises.lstat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return;
                }
                throw error;
            }
            if (stat.isDirectory()) {
                const children = yield fs_1.default.promises.readdir(path);
                yield Promise.all(children.map((child) => path_1.default.join(path, child)).map(fsExtra.rmrf));
                yield fs_1.default.promises.rmdir(path);
            }
            else {
                yield fs_1.default.promises.unlink(path);
            }
        });
    },
    outputFile(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let dir = path_1.default.dirname(path);
            yield fsExtra.mkdirp(dir);
            return fs_1.default.promises.writeFile(path, data, options);
        });
    },
    outputFileSync(path, data, options) {
        let dir = path_1.default.dirname(path);
        fsExtra.mkdirpSync(dir);
        return fs_1.default.writeFileSync(path, data, options);
    },
    outputJson(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [replacer, space] = utils_1.Is.obj(options) ? [options.replacer, options.space] : [void 0, void 0];
            return fsExtra.outputFile(path, JSON.stringify(data, replacer, space), options);
        });
    },
    outputJsonSync(path, data, options) {
        const [replacer, space] = utils_1.Is.obj(options) ? [options.replacer, options.space] : [void 0, void 0];
        return fsExtra.outputFileSync(path, JSON.stringify(data, replacer, space), options);
    },
    readJson(path, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let data = yield fs_1.default.promises.readFile(path, options);
            if (!utils_1.Is.str(data)) {
                data = data.toString('utf8');
            }
            return JSON.parse(data);
        });
    },
    readJsonSync(path, options) {
        let data = fs_1.default.readFileSync(path, options);
        if (!utils_1.Is.str(data)) {
            data = data.toString('utf8');
        }
        return JSON.parse(data);
    },
    iter(dir, 
    /** return true will skip */
    skip) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let children = yield fs_1.default.promises.readdir(dir);
            yield Promise.all(children.map((child) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let path = path_1.default.join(dir, child);
                let stat = yield fs_1.default.promises.stat(path);
                let isSkipped = yield skip(path, stat);
                if (isSkipped)
                    return;
                if (stat.isDirectory()) {
                    yield fsExtra.iter(path, skip);
                }
            })));
        });
    },
};
exports.fs = Object.assign({}, fs_1.default, fs_1.default.promises, fsExtra);
//# sourceMappingURL=fs.js.map