"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onerror = exports.after = exports.before = void 0;
exports.setGlobalOptions = setGlobalOptions;
exports.desc = desc;
exports.option = option;
exports.strict = strict;
exports.setOption = setOption;
exports.task = task;
exports.namespace = namespace;
const utils_1 = require("./utils");
const task_manager_1 = require("./task-manager");
const run_cli_1 = require("./run-cli");
/**
 * Set global options for all tasks.
 * @param options
 */
function setGlobalOptions(options) {
    Object.assign((0, task_manager_1.getGlobalTaskManager)().globalOptions, options);
}
function appendCallback(name, fn) {
    let tm = (0, task_manager_1.getGlobalTaskManager)();
    tm.listeners[name].push({
        namespaces: tm.namespaces,
        fn,
    });
}
const before = (fn) => appendCallback('before', fn);
exports.before = before;
const after = (fn) => appendCallback('after', fn);
exports.after = after;
const onerror = (fn) => appendCallback('onerror', fn);
exports.onerror = onerror;
var TaskOptions;
(function (TaskOptions) {
    TaskOptions.last = empty();
    function empty() {
        return {
            desc: undefined,
            optionDefs: [],
            strict: (0, task_manager_1.getGlobalTaskManager)().globalOptions.strict,
            loading: undefined,
        };
    }
    TaskOptions.empty = empty;
})(TaskOptions || (TaskOptions = {}));
/**
 * Define task description
 * @param desc
 */
function desc(desc) {
    TaskOptions.last.desc = desc;
}
/**
 * Define a task cli option
 * @param rawName
 * @param description
 * @param config
 */
function option(rawName, description, config) {
    TaskOptions.last.optionDefs.push([rawName, description, config]);
}
/**
 * Define task cli options are strict, which means it will throw an error if you passed undefined options.
 */
function strict() {
    TaskOptions.last.strict = true;
}
/**
 * Set options for next task.
 * @param options
 */
function setOption(options) {
    Object.assign(TaskOptions.last, options);
}
/**
 * Define a task
 * @param name
 * @param dependencies
 * @param fn
 */
function task(name, dependencies = [], fn) {
    if (utils_1.Is.fn(dependencies)) {
        fn = dependencies;
        dependencies = [];
    }
    const namespaces = (0, task_manager_1.getGlobalTaskManager)().namespaces;
    if (namespaces.length) {
        name = `${namespaces.join(':')}:${name}`;
    }
    const t = {
        name,
        namespaces,
        options: {},
        optionDefs: TaskOptions.last.optionDefs,
        desc: TaskOptions.last.desc,
        strict: TaskOptions.last.strict,
        loading: TaskOptions.last.loading,
        rawArgs: [],
        dependencies: dependencies.map(d => {
            if (utils_1.Is.str(d)) {
                return { name: d, options: {} };
            }
            else if (d._isDepBuilder) {
                return d.toTaskDep();
            }
            return d;
        }),
        fn,
    };
    TaskOptions.last = TaskOptions.empty();
    (0, task_manager_1.getGlobalTaskManager)().addTask(t);
    (0, run_cli_1.deferRunCli)();
    return t;
}
/**
 * Create namespace prefix for inner tasks
 * @param ns namespace
 * @param fn
 * @example
 * namespace('client', ns => {
 *   task('run', async ctx => {
 *     logger.log(ns) // 'client'
 *     await ctx.exec('<run cmd>')
 *   })
 * })
 * namespace('server', ns => {
 *   task('run', async ctx => {
 *     logger.log(ns) // 'server'
 *     await ctx.exec('<run cmd>')
 *   })
 * })
 *
 * ==========
 * $ yarn foy client:run
 * $ yarn foy server:run
 */
function namespace(ns, fn) {
    const tm = (0, task_manager_1.getGlobalTaskManager)();
    tm.namespaces = tm.namespaces.concat(ns);
    fn(ns);
    tm.namespaces = tm.namespaces.slice(0, -1);
}
//# sourceMappingURL=task.js.map